<?php

namespace EventPortal\Domain\Form;

use EventPortal\Contracts\HasFields;
use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Component\Value;
use EventPortal\Domain\Util\ValidationRules;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;

class Form
{
    /** @var string */
    protected $modelClass;

    /** @var null|HasFields */
    protected $model;

    /** @var Request */
    protected $request;

    /** @var array */
    public $content = [];

    /** @var array */
    public $actions = [];

    public function modelClass(string $modelClass): self
    {
        $this->modelClass = $modelClass;

        return $this;
    }

    public function model(HasFields $model): self
    {
        $this->model = $model;

        return $this;
    }

    public function addAction(callable $callable): self
    {
        $this->actions[] = $callable() instanceof Button ? $callable()->toArray() : $callable();

        return $this;
    }

    public function addSections(array $sections): self
    {
        $this->content = $sections;

        return $this;
    }

    public function withRequest(Request $request): self
    {
        $this->request = $request;

        return $this;
    }

    public function validate(): self
    {
        $this->request->validate(ValidationRules::forForm($this), $this->request->all());

        return $this;
    }

    public function submit(): HasFields
    {
        return $this->modelClass::create()
            ->initialiseFields($this->content)
            ->saveToFields($this->request->all());
    }

    public function update(): HasFields
    {
        $this->model->saveToFields($this->request->all());

        return $this->model;
    }

    public function hasModel(): bool
    {
        return $this->model !== null;
    }

    public function getModel(): ?HasFields
    {
        return $this->model;
    }

    /**
     * @return mixed|string
     */
    public function getValue(string $field)
    {
        $field = collect($this->content)
            ->pluck('children')
            ->flatten(1)
            ->firstWhere('name', $field);

        return Value::get(Arr::get($field, 'value'));
    }
}
