<?php

namespace EventPortal\Domain\Permission\Models;

use EventPortal\Contracts\HasFields;
use EventPortal\Domain\Table\GroupedActions;
use Illuminate\Database\Eloquent\Relations\MorphToMany;
use Spatie\Permission\Models\Permission as PermissionModel;

class Permission extends PermissionModel implements HasFields
{
    public function group(): MorphToMany
    {
        return $this->morphedByMany(
            config('permission.models.group'),
            'model',
            config('permission.table_names.model_has_permissions'),
            'permission_id',
            config('permission.column_names.model_morph_key')
        );
    }

    public function tableActions(): GroupedActions
    {
        $event = $this;

        return GroupedActions::make('Manage event')
            ->add(static function () use ($event): Button {
                return Button::make('View')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.show', ['event' => $event->id]);
            })
            ->add(static function () use ($event): Button {
                return Button::make('Edit')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.edit', ['event' => $event->id]);
            })
            ->add(static function (): Button {
                return Button::make('Delete')
                    ->method(InertiaMethod::VISIT())
                    ->route('admin.event.index');
            });
    }
}
