<?php

namespace EventPortal\Http\Controllers\Admin;

use EventPortal\Domain\Permission\Models\Group;
use EventPortal\Domain\Permission\Models\Permission;
use EventPortal\Domain\Permission\Models\Role;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Support\Arr;
use Inertia\Inertia;
use Inertia\Response;

class PermissionController extends Controller
{
    private function newJunkData(): void
    {
        factory(Role::class, 5)->create();
        factory(Permission::class, 25)->create();
        factory(Group::class, 5)->create();

        dd('scumbaggery complete');
    }

    public function index(): Response
    {
        // ONLY UNTIL WE HAVE PERMISSIONS TO SEED DELETE ME BEFORE DEPLOY
        // $this->newJunkData(); // Creates some garbage roles, permissions and groups

        $roles = Role::all();

        $columns = $roles->map(function (Role $role) {
            return [$role->slug => $role->name];
        });

        $groups = Group::with('permissions')->get(['id', 'name', 'slug']);

        $data = $groups->map(function (Group $group) use ($roles): array {
            return [
                'id' => $group->slug,
                'label' => $group->name,
                'children' => $group->permissions->map(function (Permission $permission) use ($roles): array {
                    return [
                        'id' => $permission->slug,
                        'label' => $permission->name,
                        'checked' => Arr::collapse($roles->map(function ($role) use ($permission): array {
                            return [$role->slug => $role->permissions->contains($permission)];
                        })),
                    ];
                }),
            ];
        });

        $breadcrumbs = Breadcrumb::make()
            ->add('Admin', route('admin.dashboard'))
            ->add('Permissions', route('admin.permission.index'))
            ->get();

        return Inertia::render('Permission/Index', [
            'data' => compact('columns', 'data'),
            'breadcrumb' => $breadcrumbs,
        ]);
    }
}
